@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -28,8 +28,8 @@
 #include "WP6FileStructure.h" 
 #include "libwpd_internal.h"
 
-WP60Header::WP60Header(GsfInput * input)
-	:	WP6Header(input)
+WP60Header::WP60Header(GsfInput * input, guint32 documentOffset, guint8 productType, guint8 fileType, guint8 majorVersion, guint8 minorVersion, guint16 documentEncryption)
+	:	WP6Header(input, documentOffset, productType, fileType, majorVersion, minorVersion, documentEncryption)
 {
 	_readIndexInformation(input);
 }
