@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2003 Net Integration Technologies
  * Copyright (C) 2002-2003 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
@@ -29,7 +29,7 @@
 #include "WPXHLListener.h"
 #include "libwpd_internal.h"
 
-#include "WP6LLParser.h" // for TableException
+#include "WP6Parser.h" // for TableException
 #include "WP6FillStylePacket.h" // for the fill packet
 
 WP6EOLGroup::WP6EOLGroup(GsfInput *input) :	
@@ -49,8 +49,8 @@ WP6EOLGroup::WP6EOLGroup(GsfInput *input) :
 
 WP6EOLGroup::~WP6EOLGroup()
 {
-	delete m_cellFgColor;
-	delete m_cellBgColor;	
+	DELETEP(m_cellFgColor);
+	DELETEP(m_cellBgColor);	
 }
 
 void WP6EOLGroup::_readContents(GsfInput *input)
@@ -155,6 +155,7 @@ void WP6EOLGroup::_readContents(GsfInput *input)
 			case WP6_EOL_GROUP_DONT_END_A_PARAGRAPH_STYLE_FOR_THIS_HARD_RETURN:
 				WPD_DEBUG_MSG(("WordPerfect: EOL Group Embedded Sub-Function: DONT_END_A_PARAGRAPH_STYLE_FOR_THIS_HARD_RETURN\n"));
 				numBytesToSkip = WP6_EOL_GROUP_DONT_END_A_PARAGRAPH_STYLE_FOR_THIS_HARD_RETURN_SIZE;
+				m_isDontEndAParagraphStyleForThisHardReturn = true;
 				break;
 			default:
 				// unsupported: shouldn't happen! an error may follow
@@ -168,85 +169,93 @@ void WP6EOLGroup::_readContents(GsfInput *input)
 	}
 }
 
-void WP6EOLGroup::parse(WP6LLListener *llListener)
+void WP6EOLGroup::parse(WP6HLListener *listener)
 {
 	WPD_DEBUG_MSG(("WordPerfect: handling an EOL group\n"));
 
 	// first off, grab any prefix information which may be useful
 	const RGBSColor * cellFgColor = m_cellFgColor;
 	const RGBSColor * cellBgColor = m_cellBgColor;
-	for (int i=0; i<getNumPrefixIDs(); i++)
+	
+	if (!cellFgColor && !cellBgColor)
 	{
-		if (const WP6FillStylePacket *fsPacket = dynamic_cast<const WP6FillStylePacket *>(llListener->getPrefixDataPacket(getPrefixIDs()[i]))) 
+		for (int i=0; i<getNumPrefixIDs(); i++)
 		{
-			cellFgColor = fsPacket->getFgColor();
-			cellBgColor = fsPacket->getBgColor();
+			if (const WP6FillStylePacket *fsPacket = dynamic_cast<const WP6FillStylePacket *>(listener->getPrefixDataPacket(getPrefixIDs()[i]))) 
+			{
+				
+				cellFgColor = fsPacket->getFgColor();
+				cellBgColor = fsPacket->getBgColor();
+			}
 		}
 	}
 
-	   
 	// main search + dispatch for messages
 	switch(getSubGroup())
 	{
-	case 0: // 0x00 (beginning of file)
-		break; // ignore
-	case WP6_EOL_GROUP_SOFT_EOL:
-	case WP6_EOL_GROUP_SOFT_EOC:
-	case WP6_EOL_GROUP_SOFT_EOC_AT_EOP: // 0x03 (soft EOC at EOP) 
-	case WP6_EOL_GROUP_DELETABLE_SOFT_EOL: // 0x014 (deletable soft EOL)
-	case WP6_EOL_GROUP_DELETABLE_SOFT_EOC: // 0x15 (deletable soft EOC) 
-	case WP6_EOL_GROUP_DELETABLE_SOFT_EOC_AT_EOP: // 0x16 (deleteable soft EOC at EOP)
-		llListener->insertCharacter((guint16) ' ');
-		break;
-	case WP6_EOL_GROUP_HARD_EOL:
-	case WP6_EOL_GROUP_HARD_EOL_AT_EOC:
-	case WP6_EOL_GROUP_HARD_EOL_AT_EOP:
-	case WP6_EOL_GROUP_DELETABLE_HARD_EOL: // 0x17 (deletable hard EOL)
-	case WP6_EOL_GROUP_DELETABLE_HARD_EOL_AT_EOC: // 0x18 (deletable hard EOL at EOC)
-	case WP6_EOL_GROUP_DELETABLE_HARD_EOL_AT_EOP: // 0x19 (deletable hard EOL at EOP)
-		llListener->insertEOL();
-		break;
-	case WP6_EOL_CHARACTER_HARD_END_OF_COLUMN: // 0x07 (hard end of column)
-		llListener->insertBreak(WPX_COLUMN_BREAK);
-		break;
-	case WP6_EOL_GROUP_HARD_EOP: // hard EOP
-	case WP6_EOL_GROUP_DELETABLE_HARD_EOP: // deletable hard EOP
-		llListener->insertBreak(WPX_PAGE_BREAK);
-		break;
-	case 0x0A: // Table Cell
-		WPD_DEBUG_MSG(("WordPerfect: EOL group: table cell\n"));
-		llListener->insertCell(m_colSpan, m_rowSpan, m_boundFromLeft, m_boundFromAbove, m_cellBorders, cellFgColor, cellBgColor);
-		break;
-	case WP6_EOL_GROUP_TABLE_ROW_AND_CELL:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_EOC:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_EOP:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC_AT_HARD_EOP:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOP:
-		WPD_DEBUG_MSG(("WordPerfect: EOL group: table row and cell\n"));
-		llListener->insertRow();
-		// the cellBorders variable already represent the cell border bits as well
-		llListener->insertCell(m_colSpan, m_rowSpan, m_boundFromLeft, m_boundFromAbove, m_cellBorders, cellFgColor, cellBgColor);
-		break;
-	case WP6_EOL_GROUP_TABLE_OFF:
-	case WP6_EOL_GROUP_TABLE_OFF_AT_EOC:
-	case WP6_EOL_GROUP_TABLE_OFF_AT_EOC_AT_EOP:
-		llListener->endTable();
-		break;
-	default: // something else we don't support yet
-		break;
+		case 0: // 0x00 (beginning of file)
+			break; // ignore
+		case WP6_EOL_GROUP_SOFT_EOL:
+		case WP6_EOL_GROUP_SOFT_EOC:
+		case WP6_EOL_GROUP_SOFT_EOC_AT_EOP: // 0x03 (soft EOC at EOP) 
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOL: // 0x17 (deletable hard EOL)
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOL_AT_EOC: // 0x18 (deletable hard EOL at EOC)
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOL_AT_EOP: // 0x19 (deletable hard EOL at EOP)
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOP: // deletable hard EOP
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOC:
+		case WP6_EOL_GROUP_DELETABLE_HARD_EOC_AT_EOP:
+			listener->insertCharacter((guint16) ' ');
+			break;
+		case WP6_EOL_GROUP_HARD_EOL:
+		case WP6_EOL_GROUP_HARD_EOL_AT_EOC:
+		case WP6_EOL_GROUP_HARD_EOL_AT_EOP:
+			listener->insertEOL();
+			break;
+		case WP6_EOL_GROUP_DELETABLE_SOFT_EOL: // 0x014 (deletable soft EOL)
+			if (m_isDontEndAParagraphStyleForThisHardReturn)
+				listener->handleLineBreak();
+			break;
+		case WP6_EOL_GROUP_HARD_EOC: // 0x07 (hard end of column)
+		case WP6_EOL_GROUP_HARD_EOC_AT_EOP:
+			listener->insertBreak(WPX_COLUMN_BREAK);
+			break;
+		case WP6_EOL_GROUP_HARD_EOP: // hard EOP
+			listener->insertBreak(WPX_PAGE_BREAK);
+			break;
+		case WP6_EOL_GROUP_TABLE_CELL: // Table Cell
+			WPD_DEBUG_MSG(("WordPerfect: EOL group: table cell\n"));
+			listener->insertCell(m_colSpan, m_rowSpan, m_boundFromLeft, m_boundFromAbove, m_cellBorders, cellFgColor, cellBgColor);
+			break;
+		case WP6_EOL_GROUP_TABLE_ROW_AND_CELL:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_EOC:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_EOP:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC_AT_HARD_EOP:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOP:
+			WPD_DEBUG_MSG(("WordPerfect: EOL group: table row and cell\n"));
+			listener->insertRow();
+			// the cellBorders variable already represent the cell border bits as well
+			listener->insertCell(m_colSpan, m_rowSpan, m_boundFromLeft, m_boundFromAbove, m_cellBorders, cellFgColor, cellBgColor);
+			break;
+		case WP6_EOL_GROUP_TABLE_OFF:
+		case WP6_EOL_GROUP_TABLE_OFF_AT_EOC:
+		case WP6_EOL_GROUP_TABLE_OFF_AT_EOC_AT_EOP:
+			listener->endTable();
+			break;
+//		case WP6_EOL_GROUP_DELETABLE_SOFT_EOC: // 0x15 (deletable soft EOC) 
+//		case WP6_EOL_GROUP_DELETABLE_SOFT_EOC_AT_EOP: // 0x16 (deleteable soft EOC at EOP)
+		default: // something else we don't support yet
+			break;
 	}
+	
 	// search for soft page breaks and dispatch messages to that effect
 	switch(getSubGroup())
 	{
-	case WP6_EOL_GROUP_SOFT_EOC_AT_EOP: // 0x03 (soft EOC at EOP) 
-	case WP6_EOL_GROUP_DELETABLE_SOFT_EOC_AT_EOP: // 0x16 (deleteable soft EOC at EOP)		
-	case WP6_EOL_GROUP_HARD_EOL_AT_EOP:
-	case WP6_EOL_GROUP_DELETABLE_HARD_EOL_AT_EOP: // 0x19 (deletable hard EOL at EOP)
-	case WP6_EOL_GROUP_TABLE_ROW_AT_EOP:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC_AT_HARD_EOP:
-	case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOP:
-	case WP6_EOL_GROUP_TABLE_OFF_AT_EOC_AT_EOP:
-		llListener->insertBreak(WPX_SOFT_PAGE_BREAK);			    
+		case WP6_EOL_GROUP_HARD_EOL_AT_EOP:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_EOP:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOC_AT_HARD_EOP:
+		case WP6_EOL_GROUP_TABLE_ROW_AT_HARD_EOP:
+		case WP6_EOL_GROUP_TABLE_OFF_AT_EOC_AT_EOP:
+			listener->insertBreak(WPX_SOFT_PAGE_BREAK);			    
 	}
 }
