@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -50,8 +50,9 @@ void WP6ExtendedDocumentSummaryPacket::_readContents(GsfInput *input)
 	m_stream = GSF_INPUT(gsf_input_memory_new(streamData, m_dataSize, TRUE));
 }
 
-void WP6ExtendedDocumentSummaryPacket::parse(WP6LLListener *llListener) const
+void WP6ExtendedDocumentSummaryPacket::parse(WP6HLListener *listener) const
 {
+	int k=0;
 	guint16 groupLength = 0;
 
 	for (int i=0; i < m_dataSize; i+=groupLength)
@@ -68,10 +69,10 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6LLListener *llListener) const
 			guint8 character = (wpChar & 0xFF);
 			guint8 characterSet = (wpChar & 0xFF00) >> 8;
 			const guint16 *chars;
-			int len, j;
+			int len;
 			len = extendedCharacterToUCS2(character,
 						      characterSet, &chars);
-			for (j = 0; j < len; j++)
+			for (int j = 0; j < len; j++)
 				name.append(chars[j]);
 		} 
 		
@@ -87,7 +88,7 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6LLListener *llListener) const
 			guint8 dayOfWeek = gsf_le_read_guint8(m_stream);
 			guint8 timeZone = gsf_le_read_guint8(m_stream);
 			guint8 unused = gsf_le_read_guint8(m_stream);
-			llListener->setDate(year, month, day, hour, minute, second, dayOfWeek, timeZone, unused);
+			listener->setDate(year, month, day, hour, minute, second, dayOfWeek, timeZone, unused);
 		}
 		else
 		{
@@ -99,13 +100,13 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6LLListener *llListener) const
 			guint8 character = (wpChar & 0xFF);
 			guint8 characterSet = (wpChar & 0xFF00) >> 8;
 			const guint16 *chars;
-			int len, j;
+			int len;
 			len = extendedCharacterToUCS2(character,
 						      characterSet, &chars);
-			for (j = 0; j < len; j++)
+			for (int j = 0; j < len; j++)
 				data.append(chars[j]);
 			} 
-			llListener->setExtendedInformation(tagID, data);
+			listener->setExtendedInformation(tagID, data);
 		}
 		WPD_CHECK_FILE_SEEK_ERROR(gsf_input_seek(m_stream, (i+groupLength), G_SEEK_SET));
 	}
