@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -27,7 +27,7 @@
 #include "WP6FontDescriptorPacket.h"
 #include "libwpd_internal.h"
 
-const char *FONT_WEIGHT_STRINGS[] = { "Black", "Bold", "Demi", "Extended", 
+const char *FONT_WEIGHT_STRINGS[] = { "Bold", "Demi", "Extended", 
 				      "Extra", "Headline", "Light", "Medium", 
 				      "Normal", "Regular", "Standaard", "Standard" };
 const char * USELESS_WP_POSTFIX = "-WP";
@@ -79,7 +79,6 @@ void WP6FontDescriptorPacket::_readContents(GsfInput *input)
    
    else 
 	   {
-		   guint16 const *tempFontName = (guint16 const *)gsf_input_read(input, sizeof(guint16)*(m_fontNameLength/2), NULL);
 		   m_fontName = new gchar[m_fontNameLength];
 
 		   guint16 tempLength=0;
@@ -87,10 +86,14 @@ void WP6FontDescriptorPacket::_readContents(GsfInput *input)
 		   int lastTokenPosition=0;
 		   for (guint16 i=0; i<(m_fontNameLength/2); i++) 
 		   {
-			   guint8 characterSet = (tempFontName[i] & 0xFF00) >> 8;
-			   guint8 character = (tempFontName[i] & 0xFF);
-			   int len, i;
+			   guint8 characterSet;
+			   guint8 character;
+			   int len;
 			   const guint16 *chars;
+			   guint16 charWord = gsf_le_read_guint16(input);
+
+			   characterSet = (charWord & 0xFF00) >> 8;
+			   character = (charWord & 0xFF);
 			   
 			   len = extendedCharacterToUCS2(character, characterSet, &chars);
 			   /* We are only using ascii characters here, and
@@ -151,4 +154,3 @@ void WP6FontDescriptorPacket::_readContents(GsfInput *input)
    WPD_DEBUG_MSG(("WordPerfect: Read Font (primary family id: %i, family member id: %i, font type: %i, font source file type: %i font name length: %i, font name: %s)\n", (int) m_primaryFamilyId, (int) m_primaryFamilyMemberId, (int) m_fontType, (int) m_fontSourceFileType, (int) m_fontNameLength, m_fontName));
 
 }
-
