@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -33,7 +33,7 @@ class WP6FootnoteEndnoteGroup : public WP6VariableLengthGroup
  public:
 	WP6FootnoteEndnoteGroup(GsfInput *input);	
 	virtual void _readContents(GsfInput *input);
-	virtual void parse(WP6LLListener *llListener);
+	virtual void parse(WP6HLListener *listener);
 };
 
 #endif /* WP6FOOTNOTEENDNOTEGROUP_H */
