@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -23,8 +23,8 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
-#include <math.h>
 #include <ctype.h>
+#include "libwpd_math.h"
 #include "WP6HLContentListener.h"
 #include "WPXHLListenerImpl.h"
 #include "WP6FileStructure.h"
@@ -35,7 +35,6 @@
 
 #include "WP6PrefixData.h"
 #include "WPXTable.h"
-#include "WPXPageSpan.h"
 
 #define WP6_DEFAULT_FONT_SIZE 12.0f
 #define WP6_DEFAULT_FONT_NAME "Times New Roman"
@@ -60,7 +59,7 @@ int _extractNumericValueFromRoman(const gchar romanChar)
 	return 1;
 }
 
-// _extractDisplayReferenceNumberFromBuf: given a number string in UCS2 represented
+// _extractDisplayReferenceNumberFromBuf: given a nuWP6_DEFAULT_FONT_SIZEmber string in UCS2 represented
 // as letters, numbers, or roman numerals.. return an integer value representing its number
 // HACK: this function is really cheesey
 // NOTE: if the input is not valid, the output is unspecified
@@ -81,6 +80,7 @@ int _extractDisplayReferenceNumberFromBuf(const UCSString &buf, const WPXNumberi
 				currentSum+=currentMark;
 			lastMark = currentMark;
 		}
+		return currentSum;
 	} 
 	else if (listType == LOWERCASE || listType == UPPERCASE)
 	{
@@ -183,53 +183,31 @@ void WP6OutlineDefinition::_updateNumberingMethods(const WP6OutlineLocation outl
 
 }
 
-_WP6ParsingState::_WP6ParsingState(bool sectionAttributesChanged) :
-	m_textAttributeBits(0),
-	m_textAttributesChanged(false),
-	m_fontSize(WP6_DEFAULT_FONT_SIZE),
-	m_fontName(g_string_new(WP6_DEFAULT_FONT_NAME)),
-	
-	m_isParagraphColumnBreak(false),
-	m_isParagraphPageBreak(false),
+_WP6ParsingState::_WP6ParsingState(WPXTableList * tableList, int nextTableIndice) :
 	m_paragraphLineSpacing(1.0f),
 	m_paragraphJustification(WPX_PARAGRAPH_JUSTIFICATION_LEFT),
 	m_tempParagraphJustification(0),
+	m_leftMargin(0.0f),
+	m_rightMargin(0.0f),
+	m_paragraphLeftMargin(0.0f),
+	m_paragraphRightMargin(0.0f),
 
-	m_isSectionOpened(false),
-
-	m_isParagraphOpened(false),
-	m_isParagraphClosed(false),
-	m_isSpanOpened(false),
-	m_numDeferredParagraphBreaks(0),
 	m_numRemovedParagraphBreaks(0),
 
+	m_tableList(tableList),
 	m_currentTable(NULL),
-	m_nextTableIndice(0),
+	m_nextTableIndice(nextTableIndice),
 	m_currentTableCol(0),
 	m_currentTableRow(0),
 	m_isTableOpened(false),
 	m_isTableRowOpened(false),
 	m_isTableCellOpened(false),
-	
-	m_isPageSpanOpened(false),
-	m_nextPageSpanIndice(0),
-	m_numPagesRemainingInSpan(0),
 
-	m_sectionAttributesChanged(sectionAttributesChanged),
-	m_numColumns(1),
-	m_marginLeft(0.0f), 
-	m_marginRight(0.0f),
-	m_pageMarginLeft(1.0f),
-	m_pageMarginRight(1.0f),
-	m_currentRow(-1),
-	m_currentColumn(-1),
-	
 	m_currentListLevel(0),
 	m_putativeListElementHasParagraphNumber(false),
 	m_putativeListElementHasDisplayReferenceNumber(false),
 
-	m_noteTextPID(0),
-	m_inSubDocument(false)
+	m_noteTextPID(0)
 {
 }
 
@@ -238,20 +216,17 @@ _WP6ParsingState::~_WP6ParsingState()
 	// fixme: erase current fontname
 }
 
-WP6HLContentListener::WP6HLContentListener(vector<WPXPageSpan *> *pageList, vector<WPXTable *> *tableList, WPXHLListenerImpl *listenerImpl) :
-	WP6HLListener(), 
-	m_listenerImpl(listenerImpl),
-	m_parseState(new WP6ParsingState),
-	m_tableList(tableList),
-	m_pageList(pageList)
+WP6HLContentListener::WP6HLContentListener(vector<WPXPageSpan *> *pageList, WPXTableList  *tableList, WPXHLListenerImpl *listenerImpl) :
+	WP6HLListener(pageList, listenerImpl), 
+	m_parseState(new WP6ParsingState(tableList))
 {
 }
 
 WP6HLContentListener::~WP6HLContentListener()
 {
-	g_string_free(m_parseState->m_fontName, TRUE);
 	typedef map<int, WP6OutlineDefinition *>::iterator Iter;
-	for (Iter outline = m_outlineDefineHash.begin(); outline != m_outlineDefineHash.end(); outline++) {
+	for (Iter outline = m_outlineDefineHash.begin(); outline != m_outlineDefineHash.end(); outline++) 
+	{
 		delete(outline->second);
 	}
 	delete m_parseState;
@@ -261,33 +236,33 @@ void WP6HLContentListener::setExtendedInformation(const guint16 type, const UCSS
 {
 	switch (type)
 	{		
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHOR):
-		m_metaData.m_author.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SUBJECT):
-		m_metaData.m_subject.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PUBLISHER):
-		m_metaData.m_publisher.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CATEGORY):
-		m_metaData.m_category.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_KEYWORDS):
-		m_metaData.m_keywords.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_LANGUAGE):
-		m_metaData.m_language.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ABSTRACT):
-		m_metaData.m_abstract.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_NAME):
-		m_metaData.m_descriptiveName.append(data);
-		break;
-	case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_TYPE):
-		m_metaData.m_descriptiveType.append(data);
-		break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHOR):
+			m_metaData.m_author.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SUBJECT):
+			m_metaData.m_subject.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PUBLISHER):
+			m_metaData.m_publisher.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CATEGORY):
+			m_metaData.m_category.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_KEYWORDS):
+			m_metaData.m_keywords.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_LANGUAGE):
+			m_metaData.m_language.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ABSTRACT):
+			m_metaData.m_abstract.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_NAME):
+			m_metaData.m_descriptiveName.append(data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_TYPE):
+			m_metaData.m_descriptiveType.append(data);
+			break;
 	}
 }
 
@@ -324,7 +299,7 @@ void WP6HLContentListener::insertCharacter(const guint16 character)
 	}
 }
 
-void WP6HLContentListener::insertTab(const guint8 tabType)
+void WP6HLContentListener::insertTab(const guint8 tabType, const float tabPosition)
 {
 	if (!isUndoOn())
 	{
@@ -335,7 +310,7 @@ void WP6HLContentListener::insertTab(const guint8 tabType)
 			// Special tabs that justify text to the right or center: only use them
 			// if we haven't started a new paragraph (this feature is a WordPerfect special)
 			// and only use them temporarily
-			if (!m_parseState->m_isParagraphOpened)
+			if (!m_ps->m_isParagraphOpened)
 			{
 
 				switch ((tabType & 0xF8) >> 3)
@@ -361,62 +336,32 @@ void WP6HLContentListener::insertTab(const guint8 tabType)
 	}
 }
 
+void WP6HLContentListener::handleLineBreak()
+{
+	if(!isUndoOn())
+	{
+		_flushText();
+		if (m_parseState->m_styleStateSequence.getCurrentState() == STYLE_BODY ||
+		    m_parseState->m_styleStateSequence.getCurrentState() == NORMAL)
+		{
+			if (!m_ps->m_isParagraphOpened)
+				_openParagraph();
+			m_listenerImpl->insertLineBreak();
+		}
+	}
+}
+
 void WP6HLContentListener::insertEOL()
 {
 	if (!isUndoOn())
 	{
 		if (m_parseState->m_styleStateSequence.getCurrentState() == NORMAL)
 			_flushText();		
-		m_parseState->m_numDeferredParagraphBreaks++; 
+		m_ps->m_numDeferredParagraphBreaks++; 
 	}
 	
 }
 
-void WP6HLContentListener::insertBreak(const guint8 breakType)
-{
-	if (!isUndoOn())
-	{	
-		_flushText();
-		switch (breakType) 
-		{
-		case WPX_COLUMN_BREAK:
-			m_parseState->m_numDeferredParagraphBreaks++;
-			m_parseState->m_isParagraphColumnBreak = true;
-			break;
-		case WPX_PAGE_BREAK:
-			m_parseState->m_numDeferredParagraphBreaks++;
-			m_parseState->m_isParagraphPageBreak = true;
-			break;
-			// TODO: (.. line break?)
-		}
-		switch (breakType)
-		{
-		case WPX_PAGE_BREAK:
-		case WPX_SOFT_PAGE_BREAK:
-			if (m_parseState->m_numPagesRemainingInSpan > 0)
-				m_parseState->m_numPagesRemainingInSpan--;
-			else
-			{
-				_openPageSpan();
-			}
-		default:
-			break;
-		}
-	}
-}
-
-void WP6HLContentListener::startDocument()
-{
-	m_listenerImpl->setDocumentMetaData(m_metaData.m_author, m_metaData.m_subject,
-					    m_metaData.m_publisher, m_metaData.m_category,
-					    m_metaData.m_keywords, m_metaData.m_language,
-					    m_metaData.m_abstract, m_metaData.m_descriptiveName,
-					    m_metaData.m_descriptiveType);
-
-	m_listenerImpl->startDocument();	
-	_openPageSpan();
-}
-
 void WP6HLContentListener::fontChange(const guint16 matchedFontPointSize, const guint16 fontPID)
 {	
 	if (!isUndoOn())
@@ -424,12 +369,12 @@ void WP6HLContentListener::fontChange(const guint16 matchedFontPointSize, const
 		// flush everything which came before this change
 		_flushText();
 
-		m_parseState->m_fontSize = rint((double)((((float)matchedFontPointSize)/100.0f)*2.0f));
+		m_ps->m_fontSize = rint((double)((((float)matchedFontPointSize)/100.0f)*2.0f));
 		const WP6FontDescriptorPacket *fontDescriptorPacket = NULL;
-		if (fontDescriptorPacket = dynamic_cast<const WP6FontDescriptorPacket *>(getPrefixDataPacket(fontPID))) {
-			g_string_printf(m_parseState->m_fontName, "%s", fontDescriptorPacket->getFontName());
+		if (fontDescriptorPacket = dynamic_cast<const WP6FontDescriptorPacket *>(WP6LLListener::getPrefixDataPacket(fontPID))) {
+			g_string_printf(m_ps->m_fontName, "%s", fontDescriptorPacket->getFontName());
 		}
-		m_parseState->m_textAttributesChanged = true;
+		m_ps->m_textAttributesChanged = true;
 	}
 }
 
@@ -484,11 +429,11 @@ void WP6HLContentListener::attributeChange(const bool isOn, const guint8 attribu
 		}
 		
 		if (isOn) 
-			m_parseState->m_textAttributeBits |= textAttributeBit;
+			m_ps->m_textAttributeBits |= textAttributeBit;
 		else
-			m_parseState->m_textAttributeBits ^= textAttributeBit;
+			m_ps->m_textAttributeBits ^= textAttributeBit;
 		
-		m_parseState->m_textAttributesChanged = true;
+		m_ps->m_textAttributesChanged = true;
 	}
 }
 
@@ -507,9 +452,11 @@ void WP6HLContentListener::justificationChange(const guint8 justification)
 		switch (justification)
 		{
 		case WP6_PARAGRAPH_JUSTIFICATION_LEFT:
-		case WP6_PARAGRAPH_JUSTIFICATION_FULL:
 			m_parseState->m_paragraphJustification = WPX_PARAGRAPH_JUSTIFICATION_LEFT;
 			break;
+		case WP6_PARAGRAPH_JUSTIFICATION_FULL:
+			m_parseState->m_paragraphJustification = WPX_PARAGRAPH_JUSTIFICATION_FULL;
+			break;
 		case WP6_PARAGRAPH_JUSTIFICATION_CENTER:
 			m_parseState->m_paragraphJustification = WPX_PARAGRAPH_JUSTIFICATION_CENTER;
 			break;
@@ -530,38 +477,67 @@ void WP6HLContentListener::marginChange(guint8 side, guint16 margin)
 {
 	if (!isUndoOn())
 	{
-		_handleLineBreakElementBegin();
-		
-		float marginInch = (float)(((double)margin + (double)WP6_NUM_EXTRA_WPU) / (double)WPX_NUM_WPUS_PER_INCH);
+		//_handleLineBreakElementBegin();
+
+		float marginInch = (float)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
 		bool marginChanged = false;
 
 		switch(side)
 		{
 		case WP6_COLUMN_GROUP_LEFT_MARGIN_SET:
-			if (m_parseState->m_marginLeft != marginInch) // FIXMEFIXME: remove this
-				m_parseState->m_sectionAttributesChanged = true;
-			m_parseState->m_marginLeft = marginInch - m_parseState->m_pageMarginLeft;
+	  	     /* Following hack is there because the paragraph and column margins are independent in WP6.
+			* A code of column margin is not canceling a code of paragraph margin and vice-versa. That is
+			* Why we keep two independent variables for current column and paragraph margins in parseState
+			* and compute the resulting m_ps->m_paragraphMarginFoo from the two each time we pass through
+			* one of marginChange or paragraphMarginChange. This will allow us also to be able to handle
+			* margin changes done by Tabs which do not last but for one paragraph. (Fridrich) */
+			m_parseState->m_leftMargin = marginInch - m_ps->m_pageMarginLeft;
+			m_ps->m_paragraphMarginLeft = m_parseState->m_leftMargin + m_parseState->m_paragraphLeftMargin;
 			break;
 		case WP6_COLUMN_GROUP_RIGHT_MARGIN_SET:
-			if (m_parseState->m_marginRight != marginInch)
-				m_parseState->m_sectionAttributesChanged = true;
-			m_parseState->m_marginRight = marginInch - m_parseState->m_pageMarginRight;
+			m_parseState->m_rightMargin = marginInch - m_ps->m_pageMarginRight;
+			m_ps->m_paragraphMarginRight = m_parseState->m_rightMargin + m_parseState->m_paragraphRightMargin;
 			break;
 		}
 
 	}	
 }
 
+void WP6HLContentListener::paragraphMarginChange(guint8 side, gint16 margin)
+{
+	if (!isUndoOn())
+	{
+		float marginInch = (float)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
+		switch(side)
+		{
+		case WPX_LEFT:
+			// This is necessary in case we have Margin Set and Left or LeftRight indentation
+			// by Tabs in the same time. The Left or LeftRight indentation applies to the
+			// current paragraph only. Margin Set applies untill an new Margin Set code.
+			m_parseState->m_paragraphLeftMargin = marginInch;
+			// Add this margin to the column margin set by "marginChange" function.
+			m_ps->m_paragraphMarginLeft = m_parseState->m_paragraphLeftMargin + m_parseState->m_leftMargin;
+			break;
+		case WPX_RIGHT:
+			m_parseState->m_paragraphRightMargin = marginInch;
+			m_ps->m_paragraphMarginRight = m_parseState->m_paragraphRightMargin + m_parseState->m_rightMargin;
+			break;
+		default:
+			break;
+		}
+	}
+}
+
 void WP6HLContentListener::columnChange(guint8 numColumns)
 {
 	if (!isUndoOn())
 	{
-		_handleLineBreakElementBegin();
-		
+		//_handleLineBreakElementBegin();
+
 		_flushText();
-		
-		m_parseState->m_numColumns = numColumns;
-		m_parseState->m_sectionAttributesChanged = true;
+
+		m_ps->m_sectionAttributesChanged = true;
+		m_ps->m_numColumns = numColumns;
 	}
 }
 
@@ -678,8 +654,8 @@ void WP6HLContentListener::styleGroupOn(const guint8 subGroup)
 			break;
 		case WP6_STYLE_GROUP_PARASTYLE_BEGIN_ON_PART2:
 			WPD_DEBUG_MSG(("WordPerfect: Handling a para style begin 2 (ON)\n"));
-			if (m_parseState->m_numDeferredParagraphBreaks > 0) {
-				m_parseState->m_numDeferredParagraphBreaks--; // very complicated: we are substituting other blocks for paragraph breaks, essentially
+			if (m_ps->m_numDeferredParagraphBreaks > 0) {
+				m_ps->m_numDeferredParagraphBreaks--; // very complicated: we are substituting other blocks for paragraph breaks, essentially
 				m_parseState->m_numRemovedParagraphBreaks = 1; // set it to 1, rather than incrementing, in case we have a leftover
 			}
 			_flushText();
@@ -709,16 +685,16 @@ void WP6HLContentListener::styleGroupOff(const guint8 subGroup)
 			m_parseState->m_styleStateSequence.setCurrentState(STYLE_BODY);      
 			if (m_parseState->m_putativeListElementHasParagraphNumber) 
 			{
-				if (m_parseState->m_sectionAttributesChanged) 
+				if (m_ps->m_sectionAttributesChanged) 
 				{
 					_openSection();
-					m_parseState->m_sectionAttributesChanged = false;
+					m_ps->m_sectionAttributesChanged = false;
 				}
 				
 				_handleListChange(m_parseState->m_currentOutlineHash);
 			}
 			else {
-				m_parseState->m_numDeferredParagraphBreaks+=m_parseState->m_numRemovedParagraphBreaks;
+				m_ps->m_numDeferredParagraphBreaks+=m_parseState->m_numRemovedParagraphBreaks;
 				m_parseState->m_numRemovedParagraphBreaks = 0;
 				_flushText();
 			}
@@ -768,13 +744,14 @@ void WP6HLContentListener::noteOff(const WPXNoteType noteType)
 		m_parseState->m_styleStateSequence.setCurrentState(NORMAL);
 		WPXNumberingType numberingType = _extractWPXNumberingTypeFromBuf(m_parseState->m_numberText, ARABIC);
 		int number = _extractDisplayReferenceNumberFromBuf(m_parseState->m_numberText, numberingType);
+		m_parseState->m_numberText.clear(); //Now, we do not need the text version of the number anymore;
 		if (noteType == FOOTNOTE)
 			m_listenerImpl->openFootnote(number);
 		else
 			m_listenerImpl->openEndnote(number);
 
 		guint16 textPID = m_parseState->m_noteTextPID;
-		_handleSubDocument(textPID);
+		handleSubDocument(textPID, false, NULL);
 
 		if (noteType == FOOTNOTE)
 			m_listenerImpl->closeFootnote();		
@@ -793,13 +770,13 @@ void WP6HLContentListener::endDocument()
 		_flushText(true); // flush the list exterior (forcing a line break, to make _flushText think we've exited a list)
 	}
 	// corner case: document contains no end of lines
-	else if (!m_parseState->m_isParagraphOpened && !m_parseState->m_isParagraphClosed)
+	else if (!m_ps->m_isParagraphOpened && !m_ps->m_isParagraphClosed)
 	{
 		_flushText();       
 	}
 	// NORMAL(ish) case document ends either inside a paragraph or outside of one,
 	// but not inside an object
-	else if (!m_parseState->m_isParagraphClosed || !m_parseState->m_isParagraphOpened)
+	else if (!m_ps->m_isParagraphClosed || !m_ps->m_isParagraphOpened)
 	{
 		_flushText();
 	}
@@ -809,6 +786,7 @@ void WP6HLContentListener::endDocument()
 
 	// close the document nice and tight
 	_closeSection();
+	_closePageSpan();
 	m_listenerImpl->endDocument();
 }
 
@@ -838,13 +816,13 @@ void WP6HLContentListener::defineTable(guint8 position, guint16 leftOffset)
 			break;
 		}
 		// Note: WordPerfect has an offset from the left edge of the page. We translate it to the offset from the left margin
-		m_tableDefinition.m_leftOffset = (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_parseState->m_marginLeft;
+		m_tableDefinition.m_leftOffset = (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
 		
 		// remove all the old column information
 		m_tableDefinition.columns.clear();
 		
 		// pull a table definition off of our stack
-		m_parseState->m_currentTable = (*m_tableList)[m_parseState->m_nextTableIndice++];
+		m_parseState->m_currentTable = (*(m_parseState->m_tableList))[m_parseState->m_nextTableIndice++];
 		m_parseState->m_currentTable->makeBordersConsistent();
 	}
 }
@@ -868,15 +846,15 @@ void WP6HLContentListener::startTable()
 {
 	if (!isUndoOn()) 
 	{		
-		_handleLineBreakElementBegin();
+		//_handleLineBreakElementBegin();
 
 		// handle corner case where we have a new section, but immediately start with a table
 		// FIXME: this isn't a very satisfying solution, and might need to be generalized
 		// as we add more table-like structures into the document
-		if (m_parseState->m_sectionAttributesChanged) 
+		if (m_ps->m_sectionAttributesChanged) 
 		{
 			_openSection();
-			m_parseState->m_sectionAttributesChanged = false;
+			m_ps->m_sectionAttributesChanged = false;
 		}
 		_openTable();
 		m_parseState->m_currentTableRow = (-1);
@@ -924,15 +902,28 @@ void WP6HLContentListener::endTable()
 // sends its text to the hll implementation and naively inserts it into the document
 // if textPID=0: Simply creates a blank paragraph
 // once finished, restores document state to what it was before
-void WP6HLContentListener::_handleSubDocument(guint16 textPID)
+void WP6HLContentListener::_handleSubDocument(guint16 textPID, const bool isHeaderFooter, WPXTableList *tableList)
 {
 	// save our old parsing state on our "stack"
 	WP6ParsingState *oldParseState = m_parseState;
-	m_parseState = new WP6ParsingState(false); // false: don't open a new section unless we must inside this type of sub-document
+	if (tableList) 
+		m_parseState = new WP6ParsingState(tableList);
+	else
+		m_parseState = new WP6ParsingState(oldParseState->m_tableList, oldParseState->m_nextTableIndice);
+
+	if (isHeaderFooter)
+	{
+		// is it is Header or Footer, assume that the initial page margins are of 1 inch.
+		// This is a behaviour that I observed with WP10 -- Fridrich
+		marginChange(WP6_COLUMN_GROUP_LEFT_MARGIN_SET, WPX_NUM_WPUS_PER_INCH);
+		marginChange(WP6_COLUMN_GROUP_RIGHT_MARGIN_SET, WPX_NUM_WPUS_PER_INCH);
+	}
+
 	if (textPID)
-		getPrefixDataPacket(textPID)->parse(this);	
+		WP6LLListener::getPrefixDataPacket(textPID)->parse(this);	
 	else
 		_openParagraph();
+
 	_flushText();
 	_closeSection();	
 
@@ -945,17 +936,17 @@ void WP6HLContentListener::_handleSubDocument(guint16 textPID)
 // _handleLineBreakElementBegin: flush everything which came before this change
 // eliminating one paragraph break which is now implicit in this change -- 
 // UNLESS the paragraph break represents something else than its name suggests, 
-// such as a paragraph or column break 
+// such as a paragraph or column break OR the paragraph break exists by itself
 // NB: I know this method is ugly. Sorry kids, the translation between WordPerfect
 // and an XMLish format is rather ugly by definition.
-void WP6HLContentListener::_handleLineBreakElementBegin() 
-{
-	if (!m_parseState->m_sectionAttributesChanged && 
-	    m_parseState->m_numDeferredParagraphBreaks > 0 &&
-	    !m_parseState->m_isParagraphColumnBreak && !m_parseState->m_isParagraphPageBreak) 
-		m_parseState->m_numDeferredParagraphBreaks--;					
-	_flushText();
-}
+// void WP6HLContentListener::_handleLineBreakElementBegin() 
+// {
+// 	if (!m_ps->m_sectionAttributesChanged && 
+// 	    m_ps->m_numDeferredParagraphBreaks > 0 &&
+// 	    !m_ps->m_isParagraphColumnBreak && !m_ps->m_isParagraphPageBreak) 
+// 		m_ps->m_numDeferredParagraphBreaks--;					
+// 	_flushText();
+// }
 
 void WP6HLContentListener::_paragraphNumberOn(const guint16 outlineHash, const guint8 level)
 {
@@ -976,39 +967,41 @@ void WP6HLContentListener::_flushText(const bool fakeText)
 	// which assumes the same condition)
 	if (m_parseState->m_styleStateSequence.getCurrentState() == NORMAL) 
 	{
-		if (m_parseState->m_currentListLevel > 0 && (m_parseState->m_numDeferredParagraphBreaks > 0 || m_parseState->m_bodyText.getLen() > 0 || fakeText) && 
+		if (m_parseState->m_currentListLevel > 0 && (m_ps->m_numDeferredParagraphBreaks > 0 || m_parseState->m_bodyText.getLen() > 0 || fakeText) && 
 		    m_parseState->m_styleStateSequence.getCurrentState() == NORMAL)
 		{
 			m_parseState->m_currentListLevel = 0;
 			_handleListChange(m_parseState->m_currentOutlineHash);
-			m_parseState->m_numDeferredParagraphBreaks--; // we have an implicit break here, when we close the list
-			m_parseState->m_isParagraphOpened = false;
+			m_ps->m_numDeferredParagraphBreaks--; // we have an implicit break here, when we close the list
+			m_ps->m_isParagraphOpened = false;
 		}
 	}
 
 	// create a new section, and a new paragraph, if our section attributes have changed and we have inserted
 	// something into the document (or we have forced a break, which assumes the same condition)
-	if (m_parseState->m_sectionAttributesChanged && (m_parseState->m_bodyText.getLen() > 0 || m_parseState->m_numDeferredParagraphBreaks > 0 || fakeText))
+	if (m_ps->m_sectionAttributesChanged && (m_parseState->m_bodyText.getLen() > 0 || m_ps->m_numDeferredParagraphBreaks > 0 || fakeText))
 	{
 		_openSection();
 		if (fakeText)
 			_openParagraph();
 	}
 
-	if (m_parseState->m_numDeferredParagraphBreaks > 0 && (m_parseState->m_styleStateSequence.getCurrentState() == NORMAL || 
+	if (m_ps->m_numDeferredParagraphBreaks > 0 && (m_parseState->m_styleStateSequence.getCurrentState() == NORMAL || 
 						 ((m_parseState->m_styleStateSequence.getCurrentState() == STYLE_BODY || 
 						   m_parseState->m_styleStateSequence.getCurrentState() == STYLE_END) &&
 						  !m_parseState->m_putativeListElementHasParagraphNumber)))
 	{
-		if (!m_parseState->m_isParagraphOpened)
-			m_parseState->m_numDeferredParagraphBreaks++;
+		if (!m_ps->m_isParagraphOpened &&
+			!(m_parseState->m_isTableOpened && !m_parseState->m_isTableCellOpened) // don't allow paragraphs to be opened when we have already opened a table, but no cell yet. - MARCM (is it really correct, or should this be fixed elsewhere??)
+		)
+			m_ps->m_numDeferredParagraphBreaks++;
 
-		while (m_parseState->m_numDeferredParagraphBreaks > 1) 
+		while (m_ps->m_numDeferredParagraphBreaks > 1) 
 			_openParagraph(); 			
 		_closeParagraph(); 
-		m_parseState->m_numDeferredParagraphBreaks = 0; // compensate for this by requiring a paragraph to be opened
+		m_ps->m_numDeferredParagraphBreaks = 0; // compensate for this by requiring a paragraph to be opened
 	}
-	else if (m_parseState->m_textAttributesChanged && (m_parseState->m_bodyText.getLen() > 0 || fakeText) && m_parseState->m_isParagraphOpened) 
+	else if (m_ps->m_textAttributesChanged && (m_parseState->m_bodyText.getLen() > 0 || fakeText) && m_ps->m_isParagraphOpened) 
 	{
 		_openSpan();
 	}
@@ -1016,8 +1009,11 @@ void WP6HLContentListener::_flushText(const bool fakeText)
 	if (m_parseState->m_bodyText.getLen() || (m_parseState->m_textBeforeNumber.getLen() && 
 						  !m_parseState->m_putativeListElementHasParagraphNumber)) 
 	{
-		if (!m_parseState->m_isParagraphOpened)
+		if (!m_ps->m_isParagraphOpened)
+		{
 			_openParagraph();
+			_openSpan();
+		}
 
 		if (m_parseState->m_textBeforeNumber.getLen() && 
 		    !m_parseState->m_putativeListElementHasParagraphNumber)
@@ -1032,7 +1028,7 @@ void WP6HLContentListener::_flushText(const bool fakeText)
 		}
 	}
 
-	m_parseState->m_textAttributesChanged = false;
+	m_ps->m_textAttributesChanged = false;
 }
 
 void WP6HLContentListener::_handleListChange(const guint16 outlineHash)
@@ -1128,70 +1124,19 @@ void WP6HLContentListener::_handleListChange(const guint16 outlineHash)
 
 void WP6HLContentListener::_openListElement()
 {
-	m_listenerImpl->openListElement(m_parseState->m_paragraphJustification, m_parseState->m_textAttributeBits,
-					m_parseState->m_marginLeft, m_parseState->m_marginRight,
-					m_parseState->m_fontName->str, m_parseState->m_fontSize, 
+	m_listenerImpl->openListElement(m_parseState->m_paragraphJustification, m_ps->m_textAttributeBits,
+					m_ps->m_paragraphMarginLeft, m_ps->m_paragraphMarginRight,
+					m_ps->m_fontName->str, m_ps->m_fontSize, 
 					m_parseState->m_paragraphLineSpacing);
-	m_parseState->m_isParagraphOpened = true; // a list element is equivalent to a paragraph
+	m_ps->m_isParagraphOpened = true; // a list element is equivalent to a paragraph
 
 }
 
-void WP6HLContentListener::_openPageSpan()
-{
-	if (m_parseState->m_isPageSpanOpened)
-		m_listenerImpl->closePageSpan();
-
-	WPXPageSpan *currentPage = (*m_pageList)[m_parseState->m_nextPageSpanIndice];
-	currentPage->makeConsistent(1);
-	bool isLastPageSpan;
-	(m_pageList->size() <= (m_parseState->m_nextPageSpanIndice+1)) ? isLastPageSpan = true : isLastPageSpan = false;	
-	
-	m_listenerImpl->openPageSpan(currentPage->getPageSpan(), isLastPageSpan,
-				     currentPage->getMarginLeft(), currentPage->getMarginRight(),
-				     currentPage->getMarginTop(), currentPage->getMarginBottom());
-
-	const vector<WPXHeaderFooter> headerFooterList = currentPage->getHeaderFooterList();
-	for (vector<WPXHeaderFooter>::const_iterator iter = headerFooterList.begin(); iter != headerFooterList.end(); iter++) 
-	{
-		if (!currentPage->getHeaderFooterSuppression((*iter).getInternalType())) 
-		{
-			m_listenerImpl->openHeaderFooter((*iter).getType(), (*iter).getOccurence());
-			_handleSubDocument((*iter).getTextPID());
-			m_listenerImpl->closeHeaderFooter((*iter).getType(), (*iter).getOccurence());					
-			WPD_DEBUG_MSG(("Header Footer Element: type: %i occurence: %i pid: %i\n", 
-				       (*iter).getType(), (*iter).getOccurence(), (*iter).getTextPID()));
-		}
-	}
-
-	m_parseState->m_pageMarginLeft = currentPage->getMarginLeft();
-	m_parseState->m_pageMarginRight = currentPage->getMarginRight();
-	m_parseState->m_numPagesRemainingInSpan = (currentPage->getPageSpan() - 1);
-	m_parseState->m_nextPageSpanIndice++;
-	m_parseState->m_isPageSpanOpened = true;
-}
-
-void WP6HLContentListener::_openSection()
-{
-	_closeSection();
-	m_listenerImpl->openSection(m_parseState->m_numColumns);	
-	m_parseState->m_sectionAttributesChanged = false;
-	m_parseState->m_isSectionOpened = true;
-}
-
-void WP6HLContentListener::_closeSection()
-{
-	_closeParagraph();
-	if (m_parseState->m_isSectionOpened)
-		m_listenerImpl->closeSection();
-
-	m_parseState->m_isSectionOpened = false;
-}
-
 void WP6HLContentListener::_openTable()
 {
 	_closeTable();
 	
-	m_listenerImpl->openTable(m_tableDefinition.m_positionBits, m_parseState->m_marginLeft, m_parseState->m_marginRight,
+	m_listenerImpl->openTable(m_tableDefinition.m_positionBits, m_ps->m_paragraphMarginLeft, m_ps->m_paragraphMarginRight,
 				  m_tableDefinition.m_leftOffset, m_tableDefinition.columns);
 	m_parseState->m_isTableOpened = true;
 }
@@ -1203,9 +1148,6 @@ void WP6HLContentListener::_closeTable()
 	if (m_parseState->m_isTableOpened)
 	{ 
 		m_listenerImpl->closeTable();
-		m_parseState->m_currentRow = 0;
-		m_parseState->m_currentColumn = 0;
-		m_parseState->m_isParagraphOpened = false;
 	}
 	m_parseState->m_isTableOpened = false;
 }
@@ -1213,8 +1155,6 @@ void WP6HLContentListener::_closeTable()
 void WP6HLContentListener::_openTableRow()
 {
 	_closeTableRow();
-	m_parseState->m_currentRow++;
-	m_parseState->m_currentColumn = -1;
 	m_listenerImpl->openTableRow();
 	m_parseState->m_isTableRowOpened = true;
 }
@@ -1233,17 +1173,16 @@ void WP6HLContentListener::_openTableCell(const guint8 colSpan, const guint8 row
 								const RGBSColor * cellFgColor, const RGBSColor * cellBgColor)
 {
 	_closeTableCell();
-	m_parseState->m_currentColumn++;
 	
 	if (!boundFromLeft && !boundFromAbove) 
 	{
-		m_listenerImpl->openTableCell(m_parseState->m_currentColumn, m_parseState->m_currentRow, colSpan, rowSpan, 
-									borderBits,
-									cellFgColor, cellBgColor);
+		m_listenerImpl->openTableCell(m_parseState->m_currentTableCol, m_parseState->m_currentTableRow, colSpan, rowSpan, 
+					      borderBits,
+					      cellFgColor, cellBgColor);
 		m_parseState->m_isTableCellOpened = true;
 	}
 	else
-		m_listenerImpl->insertCoveredTableCell(m_parseState->m_currentColumn, m_parseState->m_currentRow);
+		m_listenerImpl->insertCoveredTableCell(m_parseState->m_currentTableCol, m_parseState->m_currentTableRow);
 }
 
 void WP6HLContentListener::_closeTableCell()
@@ -1263,42 +1202,19 @@ void WP6HLContentListener::_openParagraph()
 		paragraphJustification = m_parseState->m_paragraphJustification;
 	m_parseState->m_tempParagraphJustification = 0;
 	
-	m_listenerImpl->openParagraph(paragraphJustification, m_parseState->m_textAttributeBits,
-				      m_parseState->m_marginLeft, m_parseState->m_marginRight,
-				      m_parseState->m_fontName->str, m_parseState->m_fontSize, 
+	m_listenerImpl->openParagraph(paragraphJustification, m_ps->m_textAttributeBits,
+				      m_ps->m_paragraphMarginLeft, m_ps->m_paragraphMarginRight,
+				      m_ps->m_fontName->str, m_ps->m_fontSize, 
 				      m_parseState->m_paragraphLineSpacing, 
-				      m_parseState->m_isParagraphColumnBreak, m_parseState->m_isParagraphPageBreak);
-	if (m_parseState->m_numDeferredParagraphBreaks > 0) 
-		m_parseState->m_numDeferredParagraphBreaks--;
+				      m_ps->m_isParagraphColumnBreak, m_ps->m_isParagraphPageBreak);
+	if (m_ps->m_numDeferredParagraphBreaks > 0) 
+		m_ps->m_numDeferredParagraphBreaks--;
 
-	m_parseState->m_isParagraphColumnBreak = false; 
-	m_parseState->m_isParagraphPageBreak = false;
-	m_parseState->m_isParagraphOpened = true;
-}
+	m_ps->m_isParagraphColumnBreak = false; 
+	m_ps->m_isParagraphPageBreak = false;
+	m_ps->m_isParagraphOpened = true;
 
-void WP6HLContentListener::_closeParagraph()
-{
-	_closeSpan();
-	if (m_parseState->m_isParagraphOpened)
-		m_listenerImpl->closeParagraph();	
-
-	m_parseState->m_isParagraphOpened = false;
-}
-
-void WP6HLContentListener::_openSpan()
-{
-	_closeSpan();
-	m_listenerImpl->openSpan(m_parseState->m_textAttributeBits, 
-				 m_parseState->m_fontName->str, 
-				 m_parseState->m_fontSize);	
-
-	m_parseState->m_isSpanOpened = true;
-}
-
-void WP6HLContentListener::_closeSpan()
-{
-	if (m_parseState->m_isSpanOpened)
-		m_listenerImpl->closeSpan();
-
-	m_parseState->m_isSpanOpened = false;
+	_openSpan();
+	m_ps->m_paragraphMarginLeft = m_parseState->m_leftMargin + m_parseState->m_paragraphLeftMargin;
+	m_ps->m_paragraphMarginRight = m_parseState->m_rightMargin + m_parseState->m_paragraphRightMargin;
 }
