@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -27,9 +27,10 @@
 #include "WP6LLListener.h"
 #include "libwpd_internal.h"
 
-WP6UndoGroup::WP6UndoGroup(GsfInput *input)
+WP6UndoGroup::WP6UndoGroup(GsfInput *input, guint8 groupID)
+	: WP6FixedLengthGroup(groupID)
 {
-	_read(input, WP6_UNDO_GROUP_SIZE);
+	_read(input);
 }
 
 void WP6UndoGroup::_readContents(GsfInput *input)
@@ -38,7 +39,7 @@ void WP6UndoGroup::_readContents(GsfInput *input)
 	m_undoLevel = gsf_le_read_guint16(input);
 }
 
-void WP6UndoGroup::parse(WP6LLListener *llListener)
+void WP6UndoGroup::parse(WP6HLListener *listener)
 {
-	llListener->undoChange(m_undoType, m_undoLevel);
+	listener->undoChange(m_undoType, m_undoLevel);
 }
