@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -31,8 +31,8 @@
 class WP6UndoGroup : public WP6FixedLengthGroup
 {
 public:
-	WP6UndoGroup(GsfInput *input);	
-	virtual void parse(WP6LLListener *llListener);
+	WP6UndoGroup(GsfInput *input, guint8 groupID);	
+	virtual void parse(WP6HLListener *listener);
 	
  protected:
 	virtual void _readContents(GsfInput *input);
