@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -27,12 +27,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include "WPXParser.h"
-#include "WP6LLParser.h"
-#include "WP6FileStructure.h"
+#include "WP6Parser.h"
 #include "libwpd_internal.h"
 
-WPXParser::WPXParser(GsfInput * input, WPXLLListener * listener) :
+WPXParser::WPXParser(GsfInput *input, WPXHeader *header) :
 	m_input(input),
-	m_llListener(listener)
+	m_header(header)
 {
 }
+
+WPXParser::~WPXParser()
+{
+	DELETEP(m_header);
+}
