@@ -1,4 +1,4 @@
-/* libwpd2
+/* libwpd
  * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
  *  
@@ -27,9 +27,7 @@
 #define LIBWPD_H
 
 #include "libwpd_support.h"
-#include "WPXLLListener.h"
-#include "WP6LLListener.h"
 #include "WPXHLListenerImpl.h"
-#include "WP6LLParser.h"
-#include "WP6HLParser.h"
+#include "WPDocument.h"
+
 #endif /* LIBWPD_H */
